/* evilwm - Minimalist Window Manager for X
 * Copyright (C) 1999-2000 Ciaran Anscomb <kja3@atdot.org>
 * see README for license and other details. */

#include "evilwm.h"
#include <stdlib.h>

Client *find_client(Window w, int from_parent)
{
	Client *c;

	if (w && w != root)
		for (c = head_client; c; c = c->next)
			/* if (w == (from_parent ? c->parent : c->window)) return c; */
			if (w == c->parent || w == c->window) return c;
	return NULL;
}

void set_wm_state(Client *c, int state)
{
	long data[2];

	data[0] = (long) state;
	data[1] = None; /* icon window */

	XChangeProperty(dpy, c->window, xa_wm_state, xa_wm_state,
		32, PropModeReplace, (unsigned char *)data, 2);
}

int wm_state(Client *c)
{
	Atom real_type;
	int real_format, state;
	unsigned long n, extra;
	unsigned char *data;

	if ((XGetWindowProperty(dpy, c->window, xa_wm_state, 0L, 2L, False,
				AnyPropertyType, &real_type, &real_format, &n,
				&extra, &data) == Success) && n) {
		state = *(int *)data;
		XFree(data);
		return state;
	} else return WithdrawnState;
}

void send_config(Client *c)
{
	XConfigureEvent ce;

	ce.type = ConfigureNotify;
	ce.event = c->window;
	ce.window = c->window;
	ce.x = c->x;
	ce.y = c->y;
	ce.width = c->width;
	ce.height = c->height;
	ce.border_width = 0;
	ce.above = None;
	ce.override_redirect = 0;

	XSendEvent(dpy, c->window, False, StructureNotifyMask, (XEvent *)&ce);
}

void remove_client(Client *c, int from_cleanup)
{
	Client *p;

	XGrabServer(dpy);

	if (from_cleanup) {
		if (wm_state(c) == IconicState) XMapWindow(dpy, c->window);
	} else set_wm_state(c, WithdrawnState);

	ungravitate(c);
	XRemoveFromSaveSet(dpy, c->window);
	XReparentWindow(dpy, c->window, root, c->x, c->y);
	XSetWindowBorderWidth(dpy, c->window, 1);
	XDestroyWindow(dpy, c->parent);

	if (head_client == c) head_client = c->next;
	else for (p = head_client; p && p->next; p = p->next)
		if (p->next == c) p->next = c->next;

	if (c->name) XFree(c->name);
	if (c->size) XFree(c->size);
	if (current == c)
		current = NULL;
	free(c);

	XSync(dpy, False);
	XUngrabServer(dpy);
}

void redraw(Client *c)
{
}

void change_gravity(Client *c, int multiplier)
{
	int dx = 0, dy = 0;
	int gravity = (c->size->flags & PWinGravity) ?
		c->size->win_gravity : NorthWestGravity;

	switch (gravity) {
		case NorthWestGravity: /* dx = c->border; */
		case NorthEastGravity:
		case NorthGravity: dy = c->border; break;
		/* case CenterGravity: dy = c->border; break; */
		case CenterGravity: dy = 0; break;
	}

	c->x += multiplier * dx;
	c->y += multiplier * dy;
}

void send_wm_delete(Client *c)
{
	int i, n, found = 0;
	Atom *protocols;

	if (c) {
		if (XGetWMProtocols(dpy, c->window, &protocols, &n))
			for (i=0; i<n; i++) if (protocols[i] == xa_wm_delete) found++;
		if (found) send_xmessage(c->window, xa_wm_protos, xa_wm_delete);
		else XKillClient(dpy, c->window);
	}
}

int send_xmessage(Window w, Atom a, long x)
{
	XEvent ev;

	ev.type = ClientMessage;
	ev.xclient.window = w;
	ev.xclient.message_type = a;
	ev.xclient.format = 32;
	ev.xclient.data.l[0] = x;
	ev.xclient.data.l[1] = CurrentTime;

	return XSendEvent(dpy, w, False, NoEventMask, &ev);
}

void client_to_head(Client *c) {
	Client *cs;

	for (cs = head_client; cs; cs = cs->next) {
		if (cs->next == c) {
			cs->next = c->next;
			c->next = head_client;
			head_client = c;
			return;
		}
	}
}
